package org.dgraves.DialTones;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;

/**
 * Dialog builder to create the application's error and info dialogs with
 * a "Close" button.
 * 
 * @author Dustin Graves
 */
public class MessageDialogBuilder {
	/**
	 * Creates an error dialog with a title and a message. The built-in error icon is
	 * associated with the dialog.
	 * @param context Context object for AlertDialog.Builder.
	 * @param title String specifying a title for the dialog.
	 * @param message String specifying the message to be displayed by the dialog.
	 * @return AlertDialog with the specified title and message.
	 */
    public static final AlertDialog createAlertDialog(Context context, String title, String message) {
    	return createDialog(context, title, message, android.R.drawable.ic_dialog_alert);
    }
    
	/**
	 * Creates an info dialog with a title and a message. The built-in info icon is
	 * associated with the dialog. 
	 * @param context Context object for AlertDialog.Builder.
	 * @param title String specifying a title for the dialog.
	 * @param message String specifying the message to be displayed by the dialog.
	 * @return AlertDialog with the specified title and message.
	 */
    public static final AlertDialog createInfoDialog(Context context, String title, String message) {
    	return createDialog(context, title, message, android.R.drawable.ic_dialog_info);
    }

    /**
     * Constructs a dialog with a title, message, and close button.
	 * @param context Context object for AlertDialog.Builder.
	 * @param title String specifying a title for the dialog.
	 * @param message String specifying the message to be displayed by the dialog.
     * @param icon Id for the icon to display in the dialog's title bar.
	 * @return AlertDialog with the specified title, message, and icon.
     */
    private static final AlertDialog createDialog(Context context, String title, String message, int icon) {
    	AlertDialog.Builder builder = new AlertDialog.Builder(context);
    	builder.setIcon(icon);
    	builder.setTitle(title);
    	builder.setMessage(message);
    	builder.setNegativeButton("Close", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int id) {
                dialog.cancel();
            }
    	});

    	return builder.create(); 
    }
}
