package org.dgraves.DialTones;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.SharedPreferences;
import android.media.AudioManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnKeyListener;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.CompoundButton.OnCheckedChangeListener;

public class PhoneDialerView extends Activity {
	/** AsyncTask to dial in the background. */
	private class DialTask extends AsyncTask<CharSequence, Void, String> {
		protected String doInBackground(CharSequence... numbers) {
			try {
				PhoneDialer dialer = new PhoneDialer();
				dialer.dial(numbers[0]);
				return null;
			} catch (Exception e) {
				return e.getMessage();
			}
		}
		
		protected void onPreExecute() {
			progress = ProgressDialog.show(PhoneDialerView.this, "", "Generating tones...", false);

			// Ensure that the DTMF stream is adjusted by the volume keys when progress dialog is shown
			progress.setVolumeControlStream(AudioManager.STREAM_DTMF);
		}
		
		protected void onPostExecute(String errorMessage) {
			progress.dismiss();
			if (errorMessage != null) {
				MessageDialogBuilder.createAlertDialog(PhoneDialerView.this, getString(R.string.dial_fail_title), errorMessage).show();
			}
		}
		
		ProgressDialog progress;
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Ensure that the DTMF stream is adjusted by the volume keys
		setVolumeControlStream(AudioManager.STREAM_DTMF);

		setContentView(R.layout.phone_dialer_view);

		phoneNumber = getIntent().getData().getSchemeSpecificPart();

		// Get preferences to populate view
		preferences = PreferenceManager.getDefaultSharedPreferences(this);

		// Listener to update button text when settings change
		OnCheckedChangeListener checkChangedListener = new OnCheckedChangeListener() {
			public void onCheckedChanged(CompoundButton buttonView, boolean checked) {
				setDialButtonText();
			}
		};
		
		CheckBox prefixCheck = (CheckBox)findViewById(R.id.dial_prefix_check);
		prefixCheck.setChecked(preferences.getBoolean("enable_dialPrefix", false));
		prefixCheck.setOnCheckedChangeListener(checkChangedListener);
		
		EditText prefixEdit = (EditText)findViewById(R.id.prefix_edit);
		prefixEdit.setText(preferences.getString("value_dialPrefix", getString(R.string.default_dialing_prefix)));
		prefixEdit.setOnKeyListener(new OnKeyListener() {
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if (event.getAction() == KeyEvent.ACTION_UP) {
					setDialButtonText();
				}
				return false;
			}
		});

		CheckBox oneCheck = (CheckBox)findViewById(R.id.dial_one_check);
		oneCheck.setChecked(preferences.getBoolean("enable_dialOne", false));
		oneCheck.setOnCheckedChangeListener(checkChangedListener);

		CheckBox areaCodeCheck = (CheckBox)findViewById(R.id.dial_areacode_check);
		areaCodeCheck.setChecked(preferences.getBoolean("enable_dialAreaCode", true));
		areaCodeCheck.setOnCheckedChangeListener(checkChangedListener);

		TextView dialButton = (TextView)findViewById(R.id.dial_button);
		dialButton.setText(getString(R.string.dial_button_prefix) + makeNumberToDial());
		dialButton.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				// When clicked, emit tones to dial number
				dial();
			}
		});
	}

	/** 
	 * Retrieve the dialing prefix, dialed before the phone number, from the associated EditText object.
	 * @return Prefix to dial before phone number.
	 */
	private String getDialPrefix() {
		EditText prefixEdit = (EditText)findViewById(R.id.prefix_edit);
		return prefixEdit.getText().toString();
	}

	/**
	 * Create the number to dial, based on dial settings. Prepends phone number with 
	 * a dialing prefix and '1', if the appropriate boxes are checked. Removes the
	 * area code if the appropriate box is checked.
	 * @return Phone number to dial.
	 */
	private CharSequence makeNumberToDial() {
		CheckBox oneCheck = (CheckBox)findViewById(R.id.dial_one_check);
		CheckBox prefixCheck = (CheckBox)findViewById(R.id.dial_prefix_check);
		CheckBox areaCodeCheck = (CheckBox)findViewById(R.id.dial_areacode_check);
		
		String processedNumber = phoneNumber.toString();
		if (!areaCodeCheck.isChecked()) {
			//Remove the area code
			String[] parts = processedNumber.split("\\D+");

			// The first part may be an empty string
			int index = (parts[0].length() == 0) ? 1 : 0;
			
			// Only remove the area code if the number was split into more than one part
			// Subtract index to account for an initial empty string
			if ((parts.length - index) > 1) {
				// Combine all parts after first
				StringBuffer builder = new StringBuffer(parts[++index]);
				for (int i = ++index; i < parts.length; i++) {
					builder.append("-");
					builder.append(parts[i]);
				}
				
				processedNumber = builder.toString();
			}
		}

		// Build number with prefix and dial 1 settings
		if (oneCheck.isChecked() && prefixCheck.isChecked()) {
			return getDialPrefix() + "1-" + processedNumber;
		} else if (prefixCheck.isChecked()) {
			return getDialPrefix() + processedNumber;
		} else if (oneCheck.isChecked()) {
			return "1-" + processedNumber;
		} else {
			return processedNumber;
		}
	}

	/**
	 * Sets the text of the 'button' used to activate the dialing process.  The label
	 * will always display the actual number to dial based on the current settings. 
	 */
	private void setDialButtonText() {
		TextView dialButton = (TextView)findViewById(R.id.dial_button);
		dialButton.setText(getString(R.string.dial_button_prefix) + makeNumberToDial());				
	}
	
	private void dial() {
		new DialTask().execute(makeNumberToDial());
	}

	private SharedPreferences preferences;

	private CharSequence phoneNumber;
}
